from bluer_options.env import load_config, load_env, get_env

load_env(__name__)
load_config(__name__)


ARVANCLOUD_PRIVATE_KEY = get_env("ARVANCLOUD_PRIVATE_KEY")

BLUER_VILLAGE_OBJECT = get_env("BLUER_VILLAGE_OBJECT")
BLUER_VILLAGE_TEST_OBJECT = get_env("BLUER_VILLAGE_TEST_OBJECT")



import argparse

from blueness import module
from blueness.argparse.generic import sys_exit

from bluer_sandbox import NAME
from bluer_sandbox.interview.anagram import (
    normalize_and_group_and_analyze,
    count_anagrams_in_list,
)
from bluer_sandbox.tests.test_anagram_groups import words
from bluer_sandbox.logger import logger

NAME = module.name(__file__, NAME)

parser = argparse.ArgumentParser(NAME)
parser.add_argument(
    "task",
    type=str,
    help="anagram,group",
)
parser.add_argument(
    "--query",
    type=str,
)
args = parser.parse_args()

success = False
if args.task == "anagram":
    success = True

    dict_words = [
        "listen",
        "silent",
        "enlist",
        "google",
        "elbow",
        "below",
        "bowl",
        "stressed",
        "desserts",
        "dog",
        "god",
        "odg",
    ]
    queries = ["listen", "bowl", "dessert", "god", "cat"]

    print(count_anagrams_in_list(dict_words, queries))
if args.task == "group":
    success = True
    normalize_and_group_and_analyze(words)
else:
    success = None

sys_exit(logger, NAME, args.task, success)



from blueness import module

from bluer_sandbox import NAME
from bluer_sandbox.village.village import Village
from bluer_sandbox.logger import logger


NAME = module.name(__file__, NAME)


def analyze(
    object_name: str,
    verbose: bool = False,
) -> bool:
    logger.info(f"{NAME}.analyze: {object_name}")

    village = Village()
    if not village.load(
        object_name=object_name,
        verbose=verbose,
    ):
        return False

    logger.info("🪄")

    return True



from typing import Dict

from bluer_objects.README.consts import assets2

from bluer_sandbox.README.consts import ai4k_assets2

dict_of_experiments: Dict[str, Dict] = {
    "caliper": {
        "marquee": f"{ai4k_assets2}/20251009_114411.jpg",
        "description": [
            "mechanical calipers",
            "digital calipers",
            "safety precautions",
            "measure the thickness of hair, paper, finger (what's wrong?)",
            "measure the different sides of a spoon, what else?",
            "what else?",
        ],
    },
    "multimeter": {
        "marquee": f"{ai4k_assets2}/20250616_112027.jpg",
        "description": [
            "measure the voltage of batteries, AC, battery-bus w/ different lights + on charger, what else?",
            "measure the resistance of water, metal, what else?",
            "what else?",
        ],
    },
    "ultrasonic": {
        "marquee": f"{assets2}/ultrasonic-sensor-tester/00.jpg?raw=true",
        "description": [
            "watching the film [Here's What Bat Echolocation Sounds Like, Slowed Down](https://youtu.be/qJOloliWvB8?si=_lzHkcyTP0B1S7Ba).",
            "work with the [ultrasonic sensor tester](https://github.com/kamangir/bluer-sbc/blob/main/bluer_sbc/docs/ultrasonic-sensor-tester.md), make sense of how it works, measure with one sensor.",
            "drive [arzhang](https://github.com/kamangir/bluer-ugv/tree/main/bluer_ugv/docs/arzhang) and measure how far from an obstacle it stops.",
        ],
        "items": {
            f"{assets2}/arzhang/20251005_112250.jpg?raw=true": "",
            f"{assets2}/arzhang/VID-20250830-WA0000~3_1.gif?raw=true": "",
            f"{assets2}/arzhang/VID-20250830-WA0000~3_1.gif?raw=true": "",
        },
        "cols": 2,
    },
    "ravin": {
        "marquee": f"{assets2}/ravin/20250807_103534.jpg?raw=true",
        "description": [
            "what is [ravin](https://github.com/kamangir/bluer-ugv/tree/main/bluer_ugv/docs/ravin)?",
            "what technologies make ravin work?",
            "drive ravin",
            "what are ravin's shortcomings and how can we make it better?",
        ],
        "items": {
            f"{assets2}/ravin/20250728_112123.jpg?raw=true": "",
            f"{assets2}/ravin/20250723_095022.jpg?raw=true": "",
            f"{assets2}/ravin/20250723_095155~2_1.gif?raw=true": "",
        },
        "cols": 2,
    },
    "template": {
        "marquee": "template",
        "description": [
            "",
        ],
    },
}



import numpy as np
import pygame
import time


def beep(
    frequency=440,
    duration_ms=500,
    open: bool = True,
    close: bool = True,
):
    sample_rate = 44100
    duration = duration_ms / 1000

    if open:
        pygame.mixer.init(frequency=sample_rate, size=-16, channels=1)

    if frequency is not None:
        t = np.linspace(0, duration, int(sample_rate * duration), False)
        wave = (np.sin(2 * np.pi * frequency * t) * 32767).astype(np.int16)

        sound = pygame.sndarray.make_sound(wave)
        sound.play()

    time.sleep(duration)

    if close:
        pygame.mixer.quit()


note_frequencies_basic = {
    "C4": 261.63,
    "C#4": 277.18,
    "Db4": 277.18,
    "D4": 293.66,
    "D#4": 311.13,
    "Eb4": 311.13,
    "E4": 329.63,
    "F4": 349.23,
    "F#4": 369.99,
    "Gb4": 369.99,
    "G4": 392.00,
    "G#4": 415.30,
    "Ab4": 415.30,
    "A4": 440.00,
    "A#4": 466.16,
    "Bb4": 466.16,
    "B4": 493.88,
    "C5": 523.25,
}

note_frequencies = {}

A4_freq = 440.0
A4_midi = 69

for midi_num in range(21, 109):  # MIDI 21 (A0) to MIDI 108 (C8)
    freq = A4_freq * 2 ** ((midi_num - A4_midi) / 12)

    # MIDI note name
    octave = (midi_num // 12) - 1
    note_names = ["C", "C#", "D", "D#", "E", "F", "F#", "G", "G#", "A", "A#", "B"]
    name = note_names[midi_num % 12] + str(octave)

    note_frequencies[name] = round(freq, 2)

pirates_of_caribbean = [
    "E4",
    "G4",
    "A4",
    "A4",  # "Da da da da"
    "E4",
    "G4",
    "B4",
    "B4",  # "Da da da da"
    "E4",
    "G4",
    "A4",
    "A4",  # repeat pattern
    "G4",
    "B4",
    "A4",  # descending part
    "G4",
    "E4",
    "G4",
    "A4",  # leading up again
    "B4",
    "C5",
    "D5",
    "D5",  # climax
    "E5",
    "D5",
    "C5",
    "B4",  # descending
    "A4",
    "A4",  # ending phrase
]

no_time_to_die_notes = [
    "C5",
    "B4",
    "A4",  # "I should have known..."
    "C5",
    "B4",
    "A4",  # "I'd leave alone..."
    "G4",
    "A4",
    "A4",  # "Just goes to show..."
    "G4",
    "F4",
    "E4",  # "That the blood you bleed..."
    "E4",
    "F4",
    "G4",
    "F4",  # "is just the blood you owe..."
]

pink_panther_notes = [
    # Opening sneaky motif
    "E4",
    "G4",
    "G#4",
    "A4",
    None,
    "C5",
    "B4",
    None,
    "E4",
    "G4",
    "G#4",
    "A4",
    None,
    "B4",
    "A4",
    None,
    # Cool descending phrase
    "G#4",
    None,
    "E4",
    "G#4",
    "A4",
    None,
    "C5",
    "B4",
    None,
    # Repeat with variation
    "E4",
    "G4",
    "G#4",
    "A4",
    None,
    "C5",
    "B4",
    None,
    "E4",
    "G4",
    "G#4",
    "A4",
    None,
    "B4",
    "A4",
    None,
    # Jazzy transition
    "G#4",
    "B4",
    "C5",
    None,
    "D5",
    "C5",
    "B4",
    "A4",
    # Build up with repetition
    "E4",
    "G4",
    "G#4",
    "A4",
    None,
    "C5",
    "B4",
    None,
    "E4",
    "G4",
    "G#4",
    "A4",
    None,
    "C5",
    "B4",
    None,
    # Dramatic high end
    "F5",
    None,
    "E5",
    "D#5",
    "E5",
    None,
    "C5",
    None,
    "B4",
    "A4",
    None,
]


interstellar_notes = [
    # Opening motif (slow and atmospheric)
    "C4",
    "E4",
    "G4",
    "B4",
    "G4",
    "E4",
    "C4",
    None,
    "C4",
    "E4",
    "G4",
    "B4",
    "C5",
    "B4",
    "G4",
    None,
    # Repeats with variations
    "D4",
    "F4",
    "A4",
    "C5",
    "A4",
    "F4",
    "D4",
    None,
    "D4",
    "F4",
    "A4",
    "C5",
    "D5",
    "C5",
    "A4",
    None,
    # Rhythmic development
    "E4",
    "G4",
    "B4",
    "D5",
    "B4",
    "G4",
    "E4",
    None,
    "E4",
    "G4",
    "B4",
    "D5",
    "E5",
    "D5",
    "B4",
    None,
    # Emotional peak (play slower)
    "F4",
    "A4",
    "C5",
    "E5",
    "C5",
    "A4",
    "F4",
    None,
    "F4",
    "A4",
    "C5",
    "E5",
    "F5",
    "E5",
    "C5",
    None,
]


beep(440, 500, close=False)

for note in interstellar_notes:
    freq = note_frequencies.get(note, 440)
    beep(
        freq,
        250,
        open=False,
        close=False,
    )

beep(440, 500, open=False)



from typing import List

from bluer_options.terminal import show_usage, xtra


def help_get(
    tokens: List[str],
    mono: bool,
) -> str:
    options_what = "filename | object_name | repo_name"

    options = "tiny"

    return show_usage(
        [
            "@offline_llm",
            "model",
            "get",
            f"[{options_what}]",
            f"[{options}]",
        ],
        "get model properties.",
        mono=mono,
    )


def help_download(
    tokens: List[str],
    mono: bool,
) -> str:
    options = xtra("dryrun,overwrite,tiny", mono=mono)

    return show_usage(
        [
            "@offline_llm",
            "model",
            "download",
            f"[{options}]",
        ],
        "download the model.",
        mono=mono,
    )


help_functions = {
    "download": help_download,
    "get": help_get,
}



class Person:
    def __init__(
        self,
        name: str,
        sex: str = "female",
        death: int = -1,
    ):
        self.name = name
        self.sex = sex
        self.death = death

    def as_str(
        self,
    ) -> str:
        return "{} ({}{})".format(
            self.name,
            self.sex,
            "" if self.death == -1 else f", death: {self.death}",
        )

    @property
    def is_alive(self) -> bool:
        return self.death == -1



def get(
    what: str,
    tiny: bool,
) -> str:
    if what == "filename":
        if tiny:
            return "tinyllama-1.1b-chat-v1.0.Q4_K_M.gguf"

        return "mistral-7b-instruct-v0.1.Q4_K_M.gguf"

    if what == "object_name":
        model_object_name = "offline-llm-model-object"
        if tiny:
            model_object_name = f"{model_object_name}-tiny"

        return model_object_name

    if what == "repo_name":
        if tiny:
            return "TheBloke/TinyLlama-1.1B-Chat-v1.0-GGUF"

        return "TheBloke/Mistral-7B-Instruct-v0.1-GGUF"

    return f"invalid-{what}"



import numpy as np
from typing import List, Union
import base64
from functools import reduce
from IPython.display import display, HTML
import matplotlib.pyplot as plt

from bluer_objects import storage, mlflow


def get_image_base64(filename):
    with open(filename, "rb") as f:
        data = f.read()
        return "data:image/gif;base64," + base64.b64encode(data).decode("utf-8")


def imshow(
    list_of_files: Union[
        np.ndarray,
        List[List[str]],
        List[str],
        str,
    ],
    dryrun: bool = False,
):
    if isinstance(list_of_files, np.ndarray):
        plt.figure(figsize=(10, 10))
        plt.imshow(
            list_of_files,
            cmap="viridis",
            aspect="equal",
        )
        plt.axis("off")
        plt.show()
        return

    if not isinstance(list_of_files, list):
        list_of_files = [list_of_files]
    list_of_files = [(row if isinstance(row, list) else [row]) for row in list_of_files]

    html = "".join(
        ["<table>"]
        + reduce(
            lambda x, y: x + y,
            [
                ["<tr>"]
                + [
                    '<td><img src="{}"></td>'.format(get_image_base64(filename))
                    for filename in row
                ]
                + ["</tr>"]
                for row in list_of_files
            ],
            [],
        )
        + ["</table>"]
    )

    if not dryrun:
        display(HTML(html))


def upload(
    object_name: str,
    public: bool = False,
    zip: bool = False,
) -> bool:
    if not storage.upload(
        object_name=object_name,
        public=public,
        zip=zip,
    ):
        return False

    if public or zip:
        return True

    return mlflow.log_run(object_name)



import pygame

pygame.init()

# Load piano note WAVs (e.g., "C4.wav", "D4.wav", etc.)
note_sounds = {
    "C4": pygame.mixer.Sound("piano/C4.wav"),
    "D4": pygame.mixer.Sound("piano/D4.wav"),
    # ...
}

note_sounds["C4"].play()



from typing import List

from bluer_options.terminal import show_usage, xtra


def help_analyze(
    tokens: List[str],
    mono: bool,
) -> str:
    options = xtra("~download,dryrun,upload", mono=mono)

    args = [
        "[--verbose 1]",
    ]

    return show_usage(
        [
            "@village",
            "analyze",
            f"[{options}]",
            "[.|$BLUER_VILLAGE_TEST_OBJECT]",
        ]
        + args,
        "analyze village.",
        mono=mono,
    )


help_functions = {
    "analyze": help_analyze,
}



from typing import List

from bluer_options.terminal import show_usage, xtra


def help_parse(
    tokens: List[str],
    mono: bool,
) -> str:
    options = xtra("dryrun,~upload", mono=mono)
    args = [
        "[--filename <filename>]",
    ]

    return show_usage(
        [
            "@parser",
            "parse",
            f"[{options}]",
            "<url>",
            "[-|<object-name>]",
        ]
        + args,
        "parse <url> -> <object-name>.",
        mono=mono,
    )


help_functions = {
    "parse": help_parse,
}



from typing import List, Union

from blueness import module
from bluer_options.logger import log_list
from bluer_objects import file, objects
from bluer_objects.metadata import get_from_object

from bluer_sandbox import NAME
from bluer_sandbox.village.family import Family
from bluer_sandbox.village.person import Person
from bluer_sandbox.logger import logger

NAME = module.name(__file__, NAME)


class Village:
    persons: List[Person] = []

    families: List[Family] = []

    def get_person(
        self,
        name: str,
        add: bool = False,
    ) -> Union[Person, None]:
        for person in self.persons:
            if person.name == name:
                return person

        if add:
            person = Person(name=name)
            self.persons.append(person)
            return person

        return None

    def load(
        self,
        object_name: str,
        verbose: bool = False,
    ) -> bool:
        logger.info(f"{NAME}.load({object_name})")

        # loading persons
        persons = get_from_object(object_name, "persons")
        if verbose:
            logger.info(f"persons: {persons}")

        if not isinstance(persons, dict):
            logger.error(f"persons is a {persons.__class__.__name__}, expected dict.")
            return False

        self.persons = [
            Person(
                name=name,
                sex=info.get("sex", "female"),
                death=info.get("death", -1),
            )
            for name, info in persons.items()
        ]
        log_list(
            logger,
            "loaded",
            [person.as_str() for person in self.persons],
            "person(s)",
            max_count=1000,
        )

        # loading families
        families = get_from_object(object_name, "families")
        if verbose:
            logger.info(f"families: {families}")

        if not isinstance(families, dict):
            logger.error(f"families is a {families.__class__.__name__}, expected dict.")
            return False

        self.families = [
            Family(
                parents=[
                    self.get_person(
                        name.strip(),
                        add=True,
                    )
                    for name in parents.split(" + ")
                ],
                children=[
                    self.get_person(
                        name.strip(),
                        add=True,
                    )
                    for name in info.get("children", [])
                ],
                end=info.get("end", -1),
            )
            for parents, info in families.items()
        ]

        log_list(
            logger,
            "loaded",
            [family.as_str() for family in self.families],
            "family(s)",
            max_count=1000,
        )

        log_list(
            logger,
            "created",
            [person.as_str() for person in self.persons],
            "person(s)",
            max_count=1000,
        )

        return True
